import { AuthFlowStateBase } from "../../AuthFlowState.js";
/**
 * State indicating that the MFA flow has failed.
 */
export declare class MfaFailedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
//# sourceMappingURL=MfaFailedState.d.ts.map