/*! @azure/msal-browser v4.28.2 2026-02-10 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
const InvalidUrl = "invalid_url";

export { InvalidUrl };
//# sourceMappingURL=ParsedUrlErrorCodes.mjs.map
