import { BridgeStatusCode } from "./BridgeStatusCode.js";
export type BridgeError = {
    status: BridgeStatusCode;
    code?: string;
    subError?: string;
    description?: string;
    properties?: object;
};
export declare function isBridgeError(error: unknown): error is BridgeError;
//# sourceMappingURL=BridgeError.d.ts.map