import { AccountInfo } from "./AccountInfo.js";
import { TokenResponse } from "./TokenResponse.js";
export type AuthResult = {
    token: TokenResponse;
    account: AccountInfo;
};
//# sourceMappingURL=AuthResult.d.ts.map