import { CustomAuthError } from "./CustomAuthError.js";
export declare class MethodNotImplementedError extends CustomAuthError {
    constructor(method: string, correlationId?: string);
}
//# sourceMappingURL=MethodNotImplementedError.d.ts.map