/*! @azure/msal-browser v4.28.2 2026-02-10 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
// Result type constants
const MFA_VERIFICATION_REQUIRED_RESULT_TYPE = "MfaVerificationRequiredResult";
const MFA_COMPLETED_RESULT_TYPE = "MfaCompletedResult";
function createMfaVerificationRequiredResult(input) {
    return {
        type: MFA_VERIFICATION_REQUIRED_RESULT_TYPE,
        ...input,
    };
}
function createMfaCompletedResult(input) {
    return {
        type: MFA_COMPLETED_RESULT_TYPE,
        ...input,
    };
}

export { MFA_COMPLETED_RESULT_TYPE, MFA_VERIFICATION_REQUIRED_RESULT_TYPE, createMfaCompletedResult, createMfaVerificationRequiredResult };
//# sourceMappingURL=MfaActionResult.mjs.map
