export declare const SignInScenario: {
    readonly SignInAfterSignUp: "SignInAfterSignUp";
    readonly SignInAfterPasswordReset: "SignInAfterPasswordReset";
};
export type SignInScenarioType = (typeof SignInScenario)[keyof typeof SignInScenario];
//# sourceMappingURL=SignInScenario.d.ts.map