/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export type UserAccountAttributes = Record<string, string> & {
    city?: string;
    country?: string;
    displayName?: string;
    givenName?: string;
    jobTitle?: string;
    postalCode?: string;
    state?: string;
    streetAddress?: string;
    surname?: string;
};
