/*! @azure/msal-browser v4.28.2 2026-02-10 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
// Sign in state types
const SIGN_IN_CODE_REQUIRED_STATE_TYPE = "SignInCodeRequiredState";
const SIGN_IN_PASSWORD_REQUIRED_STATE_TYPE = "SignInPasswordRequiredState";
const SIGN_IN_CONTINUATION_STATE_TYPE = "SignInContinuationState";
const SIGN_IN_COMPLETED_STATE_TYPE = "SignInCompletedState";
const SIGN_IN_FAILED_STATE_TYPE = "SignInFailedState";
// Sign up state types
const SIGN_UP_CODE_REQUIRED_STATE_TYPE = "SignUpCodeRequiredState";
const SIGN_UP_PASSWORD_REQUIRED_STATE_TYPE = "SignUpPasswordRequiredState";
const SIGN_UP_ATTRIBUTES_REQUIRED_STATE_TYPE = "SignUpAttributesRequiredState";
const SIGN_UP_COMPLETED_STATE_TYPE = "SignUpCompletedState";
const SIGN_UP_FAILED_STATE_TYPE = "SignUpFailedState";
// Reset password state types
const RESET_PASSWORD_CODE_REQUIRED_STATE_TYPE = "ResetPasswordCodeRequiredState";
const RESET_PASSWORD_PASSWORD_REQUIRED_STATE_TYPE = "ResetPasswordPasswordRequiredState";
const RESET_PASSWORD_COMPLETED_STATE_TYPE = "ResetPasswordCompletedState";
const RESET_PASSWORD_FAILED_STATE_TYPE = "ResetPasswordFailedState";
// Get account state types
const GET_ACCOUNT_COMPLETED_STATE_TYPE = "GetAccountCompletedState";
const GET_ACCOUNT_FAILED_STATE_TYPE = "GetAccountFailedState";
// Get access token state types
const GET_ACCESS_TOKEN_COMPLETED_STATE_TYPE = "GetAccessTokenCompletedState";
const GET_ACCESS_TOKEN_FAILED_STATE_TYPE = "GetAccessTokenFailedState";
// Sign out state types
const SIGN_OUT_COMPLETED_STATE_TYPE = "SignOutCompletedState";
const SIGN_OUT_FAILED_STATE_TYPE = "SignOutFailedState";
// MFA state types
const MFA_AWAITING_STATE_TYPE = "MfaAwaitingState";
const MFA_VERIFICATION_REQUIRED_STATE_TYPE = "MfaVerificationRequiredState";
const MFA_COMPLETED_STATE_TYPE = "MfaCompletedState";
const MFA_FAILED_STATE_TYPE = "MfaFailedState";
// Auth method registration (JIT) state types
const AUTH_METHOD_REGISTRATION_REQUIRED_STATE_TYPE = "AuthMethodRegistrationRequiredState";
const AUTH_METHOD_VERIFICATION_REQUIRED_STATE_TYPE = "AuthMethodVerificationRequiredState";
const AUTH_METHOD_REGISTRATION_COMPLETED_STATE_TYPE = "AuthMethodRegistrationCompletedState";
const AUTH_METHOD_REGISTRATION_FAILED_STATE_TYPE = "AuthMethodRegistrationFailedState";

export { AUTH_METHOD_REGISTRATION_COMPLETED_STATE_TYPE, AUTH_METHOD_REGISTRATION_FAILED_STATE_TYPE, AUTH_METHOD_REGISTRATION_REQUIRED_STATE_TYPE, AUTH_METHOD_VERIFICATION_REQUIRED_STATE_TYPE, GET_ACCESS_TOKEN_COMPLETED_STATE_TYPE, GET_ACCESS_TOKEN_FAILED_STATE_TYPE, GET_ACCOUNT_COMPLETED_STATE_TYPE, GET_ACCOUNT_FAILED_STATE_TYPE, MFA_AWAITING_STATE_TYPE, MFA_COMPLETED_STATE_TYPE, MFA_FAILED_STATE_TYPE, MFA_VERIFICATION_REQUIRED_STATE_TYPE, RESET_PASSWORD_CODE_REQUIRED_STATE_TYPE, RESET_PASSWORD_COMPLETED_STATE_TYPE, RESET_PASSWORD_FAILED_STATE_TYPE, RESET_PASSWORD_PASSWORD_REQUIRED_STATE_TYPE, SIGN_IN_CODE_REQUIRED_STATE_TYPE, SIGN_IN_COMPLETED_STATE_TYPE, SIGN_IN_CONTINUATION_STATE_TYPE, SIGN_IN_FAILED_STATE_TYPE, SIGN_IN_PASSWORD_REQUIRED_STATE_TYPE, SIGN_OUT_COMPLETED_STATE_TYPE, SIGN_OUT_FAILED_STATE_TYPE, SIGN_UP_ATTRIBUTES_REQUIRED_STATE_TYPE, SIGN_UP_CODE_REQUIRED_STATE_TYPE, SIGN_UP_COMPLETED_STATE_TYPE, SIGN_UP_FAILED_STATE_TYPE, SIGN_UP_PASSWORD_REQUIRED_STATE_TYPE };
//# sourceMappingURL=AuthFlowStateTypes.mjs.map
