import { AuthFlowStateBase } from "../../../core/auth_flow/AuthFlowState.js";
/**
 * Represents the completed state of the sign-in operation.
 * This state indicates that the sign-in process has finished successfully.
 */
export declare class SignInCompletedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
//# sourceMappingURL=SignInCompletedState.d.ts.map