export type TokenResponse = {
    access_token: string;
    expires_in: number;
    id_token: string;
    properties: TokenResponseProperties | null;
    scope?: string;
    shr?: string;
    extendedLifetimeToken?: boolean;
    authority?: string;
};
export type TokenResponseProperties = {
    MATS?: string;
};
//# sourceMappingURL=TokenResponse.d.ts.map