export type EncryptedData = {
    id: string;
    nonce: string;
    data: string;
    lastUpdatedAt: string;
};
export declare function isEncrypted(data: object): data is EncryptedData;
//# sourceMappingURL=EncryptedData.d.ts.map