/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

import { SignInContinuationState } from "../../../sign_in/auth_flow/state/SignInContinuationState.js";
import { RESET_PASSWORD_COMPLETED_STATE_TYPE } from "../../../core/auth_flow/AuthFlowStateTypes.js";

/**
 * Represents the state that indicates the successful completion of a password reset operation.
 */
export class ResetPasswordCompletedState extends SignInContinuationState {
    /**
     * The type of the state.
     */
    stateType = RESET_PASSWORD_COMPLETED_STATE_TYPE;
}
