/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export const PASSWORD_TOO_WEAK = "password_too_weak";
export const PASSWORD_TOO_SHORT = "password_too_short";
export const PASSWORD_TOO_LONG = "password_too_long";
export const PASSWORD_RECENTLY_USED = "password_recently_used";
export const PASSWORD_BANNED = "password_banned";
export const PASSWORD_IS_INVALID = "password_is_invalid";
export const INVALID_OOB_VALUE = "invalid_oob_value";
export const ATTRIBUTE_VALIATION_FAILED = "attribute_validation_failed";
export const NATIVEAUTHAPI_DISABLED = "nativeauthapi_disabled";
export const REGISTRATION_REQUIRED = "registration_required";
export const MFA_REQUIRED = "mfa_required";
export const PROVIDER_BLOCKED_BY_REPUTATION = "provider_blocked_by_rep";
