/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export const SIGNIN_INITIATE = "/oauth2/v2.0/initiate";
export const SIGNIN_CHALLENGE = "/oauth2/v2.0/challenge";
export const SIGNIN_TOKEN = "/oauth2/v2.0/token";
export const SIGNIN_INTROSPECT = "/oauth2/v2.0/introspect";

export const SIGNUP_START = "/signup/v1.0/start";
export const SIGNUP_CHALLENGE = "/signup/v1.0/challenge";
export const SIGNUP_CONTINUE = "/signup/v1.0/continue";

export const RESET_PWD_START = "/resetpassword/v1.0/start";
export const RESET_PWD_CHALLENGE = "/resetpassword/v1.0/challenge";
export const RESET_PWD_CONTINUE = "/resetpassword/v1.0/continue";
export const RESET_PWD_SUBMIT = "/resetpassword/v1.0/submit";
export const RESET_PWD_POLL = "/resetpassword/v1.0/poll_completion";

export const REGISTER_INTROSPECT = "/register/v1.0/introspect";
export const REGISTER_CHALLENGE = "/register/v1.0/challenge";
export const REGISTER_CONTINUE = "/register/v1.0/continue";
