import { AuthFlowStateBase } from "../../../core/auth_flow/AuthFlowState.js";
/**
 * State of a reset password operation that has failed.
 */
export declare class ResetPasswordFailedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
//# sourceMappingURL=ResetPasswordFailedState.d.ts.map