import { AuthFlowStateBase } from "../../../core/auth_flow/AuthFlowState.js";
/**
 * Represents the state of a sign-in operation that has been failed.
 */
export declare class SignInFailedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
//# sourceMappingURL=SignInFailedState.d.ts.map