import { AuthFlowActionRequiredStateBase } from "../../../core/auth_flow/AuthFlowState.js";
import { SignUpStateParameters } from "./SignUpStateParameters.js";
export declare abstract class SignUpState<TParameters extends SignUpStateParameters> extends AuthFlowActionRequiredStateBase<TParameters> {
    constructor(stateParameters: TParameters);
}
//# sourceMappingURL=SignUpState.d.ts.map