import { AuthFlowStateBase } from "../../../core/auth_flow/AuthFlowState.js";
/**
 * The completed state of the get access token flow.
 */
export declare class GetAccessTokenCompletedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
/**
 * The failed state of the get access token flow.
 */
export declare class GetAccessTokenFailedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
//# sourceMappingURL=GetAccessTokenState.d.ts.map