/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export const SignInScenario = {
    SignInAfterSignUp: "SignInAfterSignUp",
    SignInAfterPasswordReset: "SignInAfterPasswordReset",
} as const;

export type SignInScenarioType =
    (typeof SignInScenario)[keyof typeof SignInScenario];
