export declare const PASSWORD_TOO_WEAK = "password_too_weak";
export declare const PASSWORD_TOO_SHORT = "password_too_short";
export declare const PASSWORD_TOO_LONG = "password_too_long";
export declare const PASSWORD_RECENTLY_USED = "password_recently_used";
export declare const PASSWORD_BANNED = "password_banned";
export declare const PASSWORD_IS_INVALID = "password_is_invalid";
export declare const INVALID_OOB_VALUE = "invalid_oob_value";
export declare const ATTRIBUTE_VALIATION_FAILED = "attribute_validation_failed";
export declare const NATIVEAUTHAPI_DISABLED = "nativeauthapi_disabled";
export declare const REGISTRATION_REQUIRED = "registration_required";
export declare const MFA_REQUIRED = "mfa_required";
export declare const PROVIDER_BLOCKED_BY_REPUTATION = "provider_blocked_by_rep";
//# sourceMappingURL=ApiSuberrors.d.ts.map