/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export const MissingConfiguration = "missing_configuration";
export const InvalidAuthority = "invalid_authority";
export const InvalidChallengeType = "invalid_challenge_type";
