/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

import { AccountInfo } from "./AccountInfo.js";
import { TokenResponse } from "./TokenResponse.js";

export type AuthResult = {
    token: TokenResponse;
    account: AccountInfo;
};
