export type AccountInfo = {
    homeAccountId?: string;
    environment: string;
    tenantId?: string;
    username: string;
    localAccountId?: string;
    name?: string;
    idToken?: string;
    platformBrokerId?: string;
    idTokenClaims?: object;
    loginHint?: string;
};
//# sourceMappingURL=AccountInfo.d.ts.map