import { AuthFlowStateBase } from "../../../core/auth_flow/AuthFlowState.js";
/**
 * The completed state of the get account flow.
 */
export declare class GetAccountCompletedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
/**
 * The failed state of the get account flow.
 */
export declare class GetAccountFailedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType: string;
}
//# sourceMappingURL=GetAccountState.d.ts.map