/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

import { ServerTelemetryManager } from "@azure/msal-common/browser";

export type ApiRequestBase = {
    correlationId: string;
    telemetryManager: ServerTelemetryManager;
};

export type ApiResponseBase = {
    correlation_id: string;
};
