export declare const SIGNIN_INITIATE = "/oauth2/v2.0/initiate";
export declare const SIGNIN_CHALLENGE = "/oauth2/v2.0/challenge";
export declare const SIGNIN_TOKEN = "/oauth2/v2.0/token";
export declare const SIGNIN_INTROSPECT = "/oauth2/v2.0/introspect";
export declare const SIGNUP_START = "/signup/v1.0/start";
export declare const SIGNUP_CHALLENGE = "/signup/v1.0/challenge";
export declare const SIGNUP_CONTINUE = "/signup/v1.0/continue";
export declare const RESET_PWD_START = "/resetpassword/v1.0/start";
export declare const RESET_PWD_CHALLENGE = "/resetpassword/v1.0/challenge";
export declare const RESET_PWD_CONTINUE = "/resetpassword/v1.0/continue";
export declare const RESET_PWD_SUBMIT = "/resetpassword/v1.0/submit";
export declare const RESET_PWD_POLL = "/resetpassword/v1.0/poll_completion";
export declare const REGISTER_INTROSPECT = "/register/v1.0/introspect";
export declare const REGISTER_CHALLENGE = "/register/v1.0/challenge";
export declare const REGISTER_CONTINUE = "/register/v1.0/continue";
//# sourceMappingURL=CustomAuthApiEndpoint.d.ts.map