import { BridgeCapabilities } from "./BridgeCapabilities.js";
import { AccountContext } from "./BridgeAccountContext.js";
export interface InitContext {
    capabilities?: BridgeCapabilities;
    sdkName: string;
    sdkVersion: string;
    accountContext?: AccountContext;
}
//# sourceMappingURL=InitContext.d.ts.map