export interface MfaClientParametersBase {
    correlationId: string;
    continuationToken: string;
}
export interface MfaRequestChallengeParams extends MfaClientParametersBase {
    challengeType: string[];
    authMethodId: string;
}
export interface MfaSubmitChallengeParams extends MfaClientParametersBase {
    challenge: string;
    scopes: string[];
    claims?: string;
}
//# sourceMappingURL=MfaClientParameters.d.ts.map