export declare const PREFIX = "msal";
export declare const CACHE_KEY_SEPARATOR = "|";
export declare const CREDENTIAL_SCHEMA_VERSION = 2;
export declare const ACCOUNT_SCHEMA_VERSION = 2;
export declare const LOG_LEVEL_CACHE_KEY: string;
export declare const LOG_PII_CACHE_KEY: string;
export declare const BROWSER_PERF_ENABLED_KEY: string;
export declare const PLATFORM_AUTH_DOM_SUPPORT: string;
export declare const VERSION_CACHE_KEY: string;
export declare const ACCOUNT_KEYS = "account.keys";
export declare const TOKEN_KEYS = "token.keys";
export declare function getAccountKeysCacheKey(schema?: number): string;
export declare function getTokenKeysCacheKey(clientId: string, schema?: number): string;
//# sourceMappingURL=CacheKeys.d.ts.map