import { CustomAuthError } from "./CustomAuthError.js";
export declare class UserAccountAttributeError extends CustomAuthError {
    constructor(error: string, attributeName: string, attributeValue: string);
}
//# sourceMappingURL=UserAccountAttributeError.d.ts.map