/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

import { AuthFlowStateBase } from "../../AuthFlowState.js";
import { AUTH_METHOD_REGISTRATION_FAILED_STATE_TYPE } from "../../AuthFlowStateTypes.js";

/**
 * State indicating that the auth method registration flow has failed.
 */
export class AuthMethodRegistrationFailedState extends AuthFlowStateBase {
    /**
     * The type of the state.
     */
    stateType = AUTH_METHOD_REGISTRATION_FAILED_STATE_TYPE;
}
