import { SignInContinuationState } from "../../../sign_in/auth_flow/state/SignInContinuationState.js";
/**
 * Represents the state that indicates the successful completion of a password reset operation.
 */
export declare class ResetPasswordCompletedState extends SignInContinuationState {
    /**
     * The type of the state.
     */
    stateType: string;
}
//# sourceMappingURL=ResetPasswordCompletedState.d.ts.map