export type AuthBridgeResponse = string | {
    data: string;
};
export interface AuthBridge {
    addEventListener: (eventName: string, callback: (response: AuthBridgeResponse) => void) => void;
    postMessage: (message: string) => void;
    removeEventListener: (eventName: string, callback: (response: AuthBridgeResponse) => void) => void;
}
//# sourceMappingURL=AuthBridge.d.ts.map