import { CustomAuthError } from "./CustomAuthError.js";
export declare class MsalCustomAuthError extends CustomAuthError {
    constructor(error: string, errorDescription?: string, subError?: string, errorCodes?: Array<number>, correlationId?: string);
}
//# sourceMappingURL=MsalCustomAuthError.d.ts.map