/*! @azure/msal-browser v4.28.2 2026-02-10 */
'use strict';
import { AuthFlowStateBase } from '../../AuthFlowState.mjs';
import { AUTH_METHOD_REGISTRATION_FAILED_STATE_TYPE } from '../../AuthFlowStateTypes.mjs';

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * State indicating that the auth method registration flow has failed.
 */
class AuthMethodRegistrationFailedState extends AuthFlowStateBase {
    constructor() {
        super(...arguments);
        /**
         * The type of the state.
         */
        this.stateType = AUTH_METHOD_REGISTRATION_FAILED_STATE_TYPE;
    }
}

export { AuthMethodRegistrationFailedState };
//# sourceMappingURL=AuthMethodRegistrationFailedState.mjs.map
