import { CustomAuthError } from "./CustomAuthError.js";
export declare class HttpError extends CustomAuthError {
    constructor(error: string, message: string, correlationId?: string);
}
//# sourceMappingURL=HttpError.d.ts.map