import { ServerTelemetryManager } from "@azure/msal-common/browser";
export type ApiRequestBase = {
    correlationId: string;
    telemetryManager: ServerTelemetryManager;
};
export type ApiResponseBase = {
    correlation_id: string;
};
//# sourceMappingURL=ApiTypesBase.d.ts.map