import { CustomAuthError } from "./CustomAuthError.js";
export declare class InvalidArgumentError extends CustomAuthError {
    constructor(argName: string, correlationId?: string);
}
//# sourceMappingURL=InvalidArgumentError.d.ts.map