/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

// Capabilities are intended to future proof the bridge against any feature support
export interface BridgeCapabilities {
    queryAccount?: boolean;
}
