/*! @azure/msal-browser v4.28.2 2026-02-10 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
// Result type constants
const JIT_VERIFICATION_REQUIRED_RESULT_TYPE = "JitVerificationRequiredResult";
const JIT_COMPLETED_RESULT_TYPE = "JitCompletedResult";
function createJitVerificationRequiredResult(input) {
    return {
        type: JIT_VERIFICATION_REQUIRED_RESULT_TYPE,
        ...input,
    };
}
function createJitCompletedResult(input) {
    return {
        type: JIT_COMPLETED_RESULT_TYPE,
        ...input,
    };
}

export { JIT_COMPLETED_RESULT_TYPE, JIT_VERIFICATION_REQUIRED_RESULT_TYPE, createJitCompletedResult, createJitVerificationRequiredResult };
//# sourceMappingURL=JitActionResult.mjs.map
