/*! @azure/msal-browser v4.28.2 2026-02-10 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
const PASSWORD_TOO_WEAK = "password_too_weak";
const PASSWORD_TOO_SHORT = "password_too_short";
const PASSWORD_TOO_LONG = "password_too_long";
const PASSWORD_RECENTLY_USED = "password_recently_used";
const PASSWORD_BANNED = "password_banned";
const PASSWORD_IS_INVALID = "password_is_invalid";
const INVALID_OOB_VALUE = "invalid_oob_value";
const ATTRIBUTE_VALIATION_FAILED = "attribute_validation_failed";
const REGISTRATION_REQUIRED = "registration_required";
const MFA_REQUIRED = "mfa_required";
const PROVIDER_BLOCKED_BY_REPUTATION = "provider_blocked_by_rep";

export { ATTRIBUTE_VALIATION_FAILED, INVALID_OOB_VALUE, MFA_REQUIRED, PASSWORD_BANNED, PASSWORD_IS_INVALID, PASSWORD_RECENTLY_USED, PASSWORD_TOO_LONG, PASSWORD_TOO_SHORT, PASSWORD_TOO_WEAK, PROVIDER_BLOCKED_BY_REPUTATION, REGISTRATION_REQUIRED };
//# sourceMappingURL=ApiSuberrors.mjs.map
