import { CustomAuthError } from "./CustomAuthError.js";
export declare class UnexpectedError extends CustomAuthError {
    constructor(errorData: unknown, correlationId?: string);
}
//# sourceMappingURL=UnexpectedError.d.ts.map