export declare const SIGN_IN_CODE_REQUIRED_STATE_TYPE = "SignInCodeRequiredState";
export declare const SIGN_IN_PASSWORD_REQUIRED_STATE_TYPE = "SignInPasswordRequiredState";
export declare const SIGN_IN_CONTINUATION_STATE_TYPE = "SignInContinuationState";
export declare const SIGN_IN_COMPLETED_STATE_TYPE = "SignInCompletedState";
export declare const SIGN_IN_FAILED_STATE_TYPE = "SignInFailedState";
export declare const SIGN_UP_CODE_REQUIRED_STATE_TYPE = "SignUpCodeRequiredState";
export declare const SIGN_UP_PASSWORD_REQUIRED_STATE_TYPE = "SignUpPasswordRequiredState";
export declare const SIGN_UP_ATTRIBUTES_REQUIRED_STATE_TYPE = "SignUpAttributesRequiredState";
export declare const SIGN_UP_COMPLETED_STATE_TYPE = "SignUpCompletedState";
export declare const SIGN_UP_FAILED_STATE_TYPE = "SignUpFailedState";
export declare const RESET_PASSWORD_CODE_REQUIRED_STATE_TYPE = "ResetPasswordCodeRequiredState";
export declare const RESET_PASSWORD_PASSWORD_REQUIRED_STATE_TYPE = "ResetPasswordPasswordRequiredState";
export declare const RESET_PASSWORD_COMPLETED_STATE_TYPE = "ResetPasswordCompletedState";
export declare const RESET_PASSWORD_FAILED_STATE_TYPE = "ResetPasswordFailedState";
export declare const GET_ACCOUNT_COMPLETED_STATE_TYPE = "GetAccountCompletedState";
export declare const GET_ACCOUNT_FAILED_STATE_TYPE = "GetAccountFailedState";
export declare const GET_ACCESS_TOKEN_COMPLETED_STATE_TYPE = "GetAccessTokenCompletedState";
export declare const GET_ACCESS_TOKEN_FAILED_STATE_TYPE = "GetAccessTokenFailedState";
export declare const SIGN_OUT_COMPLETED_STATE_TYPE = "SignOutCompletedState";
export declare const SIGN_OUT_FAILED_STATE_TYPE = "SignOutFailedState";
export declare const MFA_AWAITING_STATE_TYPE = "MfaAwaitingState";
export declare const MFA_VERIFICATION_REQUIRED_STATE_TYPE = "MfaVerificationRequiredState";
export declare const MFA_COMPLETED_STATE_TYPE = "MfaCompletedState";
export declare const MFA_FAILED_STATE_TYPE = "MfaFailedState";
export declare const AUTH_METHOD_REGISTRATION_REQUIRED_STATE_TYPE = "AuthMethodRegistrationRequiredState";
export declare const AUTH_METHOD_VERIFICATION_REQUIRED_STATE_TYPE = "AuthMethodVerificationRequiredState";
export declare const AUTH_METHOD_REGISTRATION_COMPLETED_STATE_TYPE = "AuthMethodRegistrationCompletedState";
export declare const AUTH_METHOD_REGISTRATION_FAILED_STATE_TYPE = "AuthMethodRegistrationFailedState";
//# sourceMappingURL=AuthFlowStateTypes.d.ts.map