/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export const NoNetworkConnectivity = "no_network_connectivity";
export const FailedSendRequest = "failed_send_request";
