/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

import { AuthenticationMethod } from "../../network_client/custom_auth_api/types/ApiResponseTypes.js";

/**
 * Details for an authentication method to be registered.
 */
export interface AuthMethodDetails {
    /**
     * The authentication method type to register.
     */
    authMethodType: AuthenticationMethod;

    /**
     * The verification contact (email, phone number) for the authentication method.
     */
    verificationContact: string;
}
