export declare const SIGN_IN_WITH_CODE_START = 100001;
export declare const SIGN_IN_WITH_PASSWORD_START = 100002;
export declare const SIGN_IN_SUBMIT_CODE = 100003;
export declare const SIGN_IN_SUBMIT_PASSWORD = 100004;
export declare const SIGN_IN_RESEND_CODE = 100005;
export declare const SIGN_IN_AFTER_SIGN_UP = 100006;
export declare const SIGN_IN_AFTER_PASSWORD_RESET = 100007;
export declare const SIGN_UP_WITH_PASSWORD_START = 100021;
export declare const SIGN_UP_START = 100022;
export declare const SIGN_UP_SUBMIT_CODE = 100023;
export declare const SIGN_UP_SUBMIT_PASSWORD = 100024;
export declare const SIGN_UP_SUBMIT_ATTRIBUTES = 100025;
export declare const SIGN_UP_RESEND_CODE = 100026;
export declare const PASSWORD_RESET_START = 100041;
export declare const PASSWORD_RESET_SUBMIT_CODE = 100042;
export declare const PASSWORD_RESET_SUBMIT_PASSWORD = 100043;
export declare const PASSWORD_RESET_RESEND_CODE = 100044;
export declare const ACCOUNT_GET_ACCOUNT = 100061;
export declare const ACCOUNT_SIGN_OUT = 100062;
export declare const ACCOUNT_GET_ACCESS_TOKEN = 100063;
export declare const JIT_CHALLENGE_AUTH_METHOD = 100081;
export declare const JIT_SUBMIT_CHALLENGE = 100082;
export declare const MFA_REQUEST_CHALLENGE = 100101;
export declare const MFA_SUBMIT_CHALLENGE = 100102;
//# sourceMappingURL=PublicApiId.d.ts.map