/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

export const CONTINUATION_TOKEN_MISSING = "continuation_token_missing";
export const INVALID_RESPONSE_BODY = "invalid_response_body";
export const EMPTY_RESPONSE = "empty_response";
export const UNSUPPORTED_CHALLENGE_TYPE = "unsupported_challenge_type";
export const ACCESS_TOKEN_MISSING = "access_token_missing";
export const ID_TOKEN_MISSING = "id_token_missing";
export const REFRESH_TOKEN_MISSING = "refresh_token_missing";
export const INVALID_EXPIRES_IN = "invalid_expires_in";
export const INVALID_TOKEN_TYPE = "invalid_token_type";
export const HTTP_REQUEST_FAILED = "http_request_failed";
export const INVALID_REQUEST = "invalid_request";
export const USER_NOT_FOUND = "user_not_found";
export const INVALID_GRANT = "invalid_grant";
export const CREDENTIAL_REQUIRED = "credential_required";
export const ATTRIBUTES_REQUIRED = "attributes_required";
export const USER_ALREADY_EXISTS = "user_already_exists";
export const INVALID_POLL_STATUS = "invalid_poll_status";
export const PASSWORD_CHANGE_FAILED = "password_change_failed";
export const PASSWORD_RESET_TIMEOUT = "password_reset_timeout";
export const CLIENT_INFO_MISSING = "client_info_missing";
export const EXPIRED_TOKEN = "expired_token";
export const ACCESS_DENIED = "access_denied";
