/**
 * Utility function to remove an element from an array in place.
 * @param array - The array from which to remove the element.
 * @param element - The element to remove from the array.
 */
export declare function removeElementFromArray(array: Array<string>, element: string): void;
//# sourceMappingURL=Helpers.d.ts.map