/*! @azure/msal-browser v4.28.2 2026-02-10 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Represents an HTTP method type.
 */
const HttpMethod = {
    GET: "GET",
    POST: "POST"};

export { HttpMethod };
//# sourceMappingURL=IHttpClient.mjs.map
